#pragma once

namespace esphome {
namespace dashboard_import {

const char *get_package_import_url();
void set_package_import_url(const char *url);

}  // namespace dashboard_import
}  // namespace esphome
